"""Model to load data and save data."""

# Import built-in modules
import json
import os

# Import built-in modules
from smartEdit import helper
from smartEdit import templates


class Model(object):
    """Model to load data and save data."""

    # Absolute path of the settings file.
    _SETTINGS_FILE = os.path.join(helper.get_package_dir(), "settings.json")

    # Absolute path of the knobs file that stores general quickedit knobs to
    # always expose per node class.
    _CLASS_KNOBS_FILE = os.path.join(helper.get_package_dir(), "knobs.json")

    def __init__(self, node_class=""):
        """Initialize the Model instance.

        Args:
            node_class (str): Name of the node class.

        """
        self.node_class = node_class

        self.ensure_settings_dir()
        self.ensure_settings_file()
        self.ensure_knobs_file()
        self.settings = self.load_settings()
        self.ensure_quickscripts_root()
        self.class_knobs = self.load_class_knobs()
        self.quick_scripts = self.load_quick_scripts()

    def load_quick_scripts(self):
        """Load the quickscripts.rst for the current node class.

        Returns:
            :obj:`list` of :obj:`str`: absolute paths of all active
                quickscripts.rst for the current node class.

        """
        quick_scripts_dir = os.path.join(self.settings["quickcripts_root"],
                                         self.node_class)

        if not os.path.isdir(quick_scripts_dir):
            return []

        return [os.path.join(quick_scripts_dir, name)
                for name in os.listdir(quick_scripts_dir)
                if name.endswith(".py")
                and not name.startswith("_")]

    def reload_quickscripts(self):
        """Reload quickscripts.rst to get all latest quickscripts.rst."""
        self.quick_scripts = self.load_quick_scripts()

    def reload_class_knobs(self):
        """Reload the class knobs."""
        self.class_knobs = self.load_class_knobs()

    def ensure_settings_dir(self):
        """Ensure the settings directory."""
        settings_dir = os.path.dirname(self._SETTINGS_FILE)
        if not os.path.isdir(settings_dir):
            os.makedirs(settings_dir)

    def ensure_settings_file(self):
        """Ensure the settings file exists."""
        if not os.path.isfile(self._SETTINGS_FILE):
            with open(self._SETTINGS_FILE, "w") as file_:
                json.dump(templates.SMART_EDIT_SETTINGS, file_, indent=4)

    def ensure_knobs_file(self):
        """Ensure the knobs file exists."""
        if not os.path.isfile(self._CLASS_KNOBS_FILE):
            with open(self._CLASS_KNOBS_FILE, "w") as file_:
                json.dump(templates.CLASS_KNOBS, file_, indent=4)

    def ensure_quickscripts_root(self):
        """Ensure the quickscripts.rst directory exists."""
        dir_ = self.settings["quickcripts_root"]
        if not os.path.isdir(dir_):
            os.makedirs(dir_)

    def load_settings(self):
        """Load settings."""
        return helper.load_settings()

    def save_settings(self, id_, value):
        """Save the new value under the given id.

        Args:
            id_ (str): Identifier of the setting to update.
            value (multiple): The new value to update the given id to.

        """
        self.settings[id_] = value
        with open(self._SETTINGS_FILE, "w") as file_:
            json.dump(self.settings, file_, indent=4)

    def load_class_knobs(self):
        """Load the class knobs file."""
        with open(self._CLASS_KNOBS_FILE, "r") as file_:
            return json.load(file_)

    def save_to_class_knobs(self, node_class, knob_name):
        """Save the knob name to the given node class if not present."""
        if not self.class_knobs.get(node_class):
            self.class_knobs[node_class] = []

        if not knob_name in self.class_knobs[node_class]:
            self.class_knobs[node_class].append(knob_name)

        with open(self._CLASS_KNOBS_FILE, "w") as file_:
            json.dump(self.class_knobs, file_, indent=4)

    def remove_class_knob(self, node_class, knob_name):
        """Remove the given knob name from the given node class.

        Args:
            node_class (str): Name of the node class to remove from.
            knob_name (str): Name of the knob to remove from the node class.

        """
        if not self.class_knobs.get(node_class):
            return

        if knob_name in self.class_knobs[node_class]:
            self.class_knobs[node_class].remove(knob_name)

        with open(self._CLASS_KNOBS_FILE, "w") as file_:
            json.dump(self.class_knobs, file_, indent=4)
