"""Controller to drive the view."""

# Import built-in modules
import imp
import os
import re

# PySide import switch
try:
    from PySide import QtGui as QtWidgets
    from PySide import QtGui
    from PySide import QtCore
except ImportError as error:
    from PySide2 import QtWidgets
    from PySide2 import QtGui
    from PySide2 import QtCore

# Import local modules
from smartEdit import helper
from smartEdit import knob_mapper
from smartEdit import nuke_funcs
from smartEdit.constants import NO_MAPPING_FOUND
from smartEdit.mvc import dialogs
from smartEdit.mvc import gui_helper
from smartEdit.mvc.tooltips import TOOLTIPS
from smartEdit.widgets import factory

# Dialog to add a knob as quickedit so that it is available via smartEdit.
_ADD_KNOB_DIALOG = None

# Icons for this package.
_ICONS = gui_helper.load_icons()


class Controller(object):
    """Drive the view."""

    def __init__(self, view, model, node):
        """Initialize the Controller instance.

        Args:
            view (smartEdit.mvc.view.Main): The view to drive.
            model (smartEdit.mvc.model.Mode): the model to load and save data.
            node (nuke.Node): The node to edit.

        """
        self.view = view
        self.model = model
        self.node = node

        self.view.setWindowTitle("smartEdit {}".format(self.node.name()))
        gui_helper.move_widget_under_cursor(self.view)

        self.settings = self.model.load_settings()
        self.node_class_knobs = \
            self.model.class_knobs.get(self.node.Class()) or []

        self.setup_widgets()
        self.create_signals()
        self.populate_knobs()

        self.view.input_keep.setFocus()

        self.setup_settings_widgets()
        self.set_window_opacity(int(self.settings["opacity"]))

        if self.settings["show_tooltips"]:
            self.create_tooltips()

    def create_signals(self):
        """Create signals."""
        self.view.key_pressed.connect(lambda mode: self.handle_keys(mode))

        self.view.key_released.connect(lambda mode: self.handle_keys(mode))
        self.view.button_plus.clicked.connect(lambda: self.show_add_knob())
        self.view.button_pin.clicked.connect(lambda: self.toggle_pinned())
        self.view.settings.slider.sliderMoved.connect(
            lambda: self.update_window_opacity_values())
        self.view.settings.input_slider_percentage.returnPressed.connect(
            lambda: self.update_window_opacity_values(slider=False))
        self.view.button_settings.clicked.connect(
            lambda: self.toggle_settings())
        self.view.settings.button_save.clicked.connect(
            lambda: self.save_settings())
        self.view.button_plus.knob_dropped.connect(
            lambda text: self.add_knob(text))

    def create_tooltips(self):
        """Create tooltips."""
        self.view.button_pin.setToolTip(TOOLTIPS["pin"])
        self.view.button_settings.setToolTip(TOOLTIPS["settings"])
        self.view.button_plus.setToolTip(TOOLTIPS["add"])
        self.view.settings.check_show_tooltips.setToolTip(
            TOOLTIPS["show_tooltips"])
        self.view.settings.check_pin.setToolTip(TOOLTIPS["pin"])
        self.view.settings.check_frameless.setToolTip(TOOLTIPS["frameless"])
        self.view.settings.check_advanced_widgets.setToolTip(
            TOOLTIPS["advanced_widgets"])
        self.view.settings.check_enable_add_for_node_class.setToolTip(
            TOOLTIPS["enable_add_for_node_class"])
        self.view.settings.label_slider_precision.setToolTip(
            TOOLTIPS["slider_precision"])
        self.view.settings.spin_slider_precision.setToolTip(
            TOOLTIPS["slider_precision"])
        self.view.settings.slider.setToolTip(TOOLTIPS["window_opacity"])
        self.view.settings.label_slider.setToolTip(TOOLTIPS["window_opacity"])
        self.view.settings.input_slider_percentage.setToolTip(
            TOOLTIPS["window_opacity"])
        self.view.settings.text_ignore.setToolTip(TOOLTIPS["ignore_knobs"])
        self.view.settings.label_hotkey.setToolTip(TOOLTIPS["hotkey"])
        self.view.settings.input_hotkey.setToolTip(TOOLTIPS["hotkey"])

    def add_knob(self, text):
        """Register dropped knob."""
        pattern = r"\:nuke\:knob\:[\w]+\.(?P<knob>[\w]+)"

        match = re.match(pattern, text)

        if not match:
            return

        knob = match.groupdict()["knob"]

        self.show_add_knob(selected_knob=knob)

    def save_settings(self):
        """Save settings and toggle settings widget visibility."""
        entered_hotkey = self.view.settings.input_hotkey.text().upper()

        if not hasattr(QtCore.Qt, "Key_{}".format(entered_hotkey)):
            message = "Invalid character. Please use a different hotkey."
            dialogs.show_message_box(self.view, message)
            return

        self.model.save_settings("hotkey", entered_hotkey)

        self.toggle_settings()
        self.view.adjustSize()

        self.model.save_settings(
            "opacity", int(self.view.settings.input_slider_percentage.text()))

        knobs_ignore = self.view.settings.text_ignore.toPlainText()
        knobs_ignore = knobs_ignore.replace(" ", "")
        knobs_ignore = knobs_ignore.split(",")
        knobs_ignore = [knob.lower() for knob in knobs_ignore]
        self.model.save_settings("knobs_ignore", knobs_ignore)

        self.model.save_settings(
            "pin", self.view.settings.check_pin.isChecked())

        self.model.save_settings(
            "show_tooltips",
            self.view.settings.check_show_tooltips.isChecked())

        self.model.save_settings(
            "frameless",
            self.view.settings.check_frameless.isChecked())

        self.model.save_settings(
            "advanced_widgets",
            self.view.settings.check_advanced_widgets.isChecked())

        self.model.save_settings(
            "enable_add_for_node_class",
            self.view.settings.check_enable_add_for_node_class.isChecked()
        )

        self.model.save_settings(
            "slider_precision",
            self.view.settings.spin_slider_precision.value())

    def toggle_settings(self):
        """Toggle the settings widget."""
        self.view.settings.setVisible(not self.view.settings.isVisible())
        self.view.group_main.setVisible(not self.view.settings.isVisible())

    def update_window_opacity_values(self, slider=True):
        """Update the opacity of the window and the percentage widget.

        Args:
            slider (bool, optional): If True trigger signal from slider and
                update input, otherwise trigger from input and update slider.

        """
        if slider:
            value = self.view.settings.slider.value()
            self.view.settings.input_slider_percentage.setText(str(value))
        else:
            value = int(self.view.settings.input_slider_percentage.text())
            if value < 30:
                value = 40
            if value > 100:
                value = 100
            self.view.settings.slider.setValue(value)
            self.view.settings.input_slider_percentage.setText(str(value))

        self.set_window_opacity(value)

    def set_window_opacity(self, value):
        """Set the window opacity.

        Args:
            value (int) Percentage value to use. Normalized to 1 - 100.

        """
        slider_value_dec = value * 0.01
        self.view.setWindowOpacity(slider_value_dec)

    def toggle_pinned(self):
        """Toggle between pinned and unpinned mode and toggle pin icon."""
        self.settings["pin"] = not self.settings["pin"]
        self.model.save_settings("pin", self.settings["pin"] )

        if self.settings["pin"]:
            icon = _ICONS["pin_pinned"]
        else:
            icon = _ICONS["pin_unpinned"]

        gui_helper.set_icon(self.view.button_pin, icon)

    def setup_widgets(self):
        """Set up widgets."""
        pinned = self.settings["pin"]
        if pinned:
            icon = _ICONS["pin_pinned"]
        else:
            icon = _ICONS["pin_unpinned"]

        gui_helper.set_icon(self.view.button_pin, icon)

    def setup_settings_widgets(self):
        """Set up settings widgets based on current config."""
        self.view.settings.check_show_tooltips.setChecked(
            self.settings["show_tooltips"])

        self.view.settings.check_frameless.setChecked(
            self.settings["frameless"])

        self.view.settings.check_pin.setChecked(
            self.settings["pin"])

        self.view.settings.check_advanced_widgets.setChecked(
            self.settings["advanced_widgets"])

        self.view.settings.check_enable_add_for_node_class.setChecked(
            self.settings["enable_add_for_node_class"]
        )

        self.view.settings.spin_slider_precision.setValue(
            self.settings["slider_precision"])

        self.view.settings.slider.setSliderPosition(
            self.settings["opacity"])

        self.view.settings.input_slider_percentage.setText(
            str(self.settings["opacity"]))

        self.view.settings.input_hotkey.setText(
            self.settings["hotkey"])

        knobs_ignore = ", ".join(self.settings["knobs_ignore"])
        self.view.settings.text_ignore.setText(knobs_ignore)

    def populate_knobs(self):
        """Add quickedit knobs for the node."""
        gui_helper.flush_layout(self.view.layout_widgets)

        # Add quickscripts.rst.
        if self.model.quick_scripts:
            for quick_script in self.model.quick_scripts:
                self.create_quick_script_button(quick_script)

            self.add_spacer()

        # Add quickedit knobs for global knobs that should always be exposed.
        if self.node_class_knobs:
            for knob_name in self.node_class_knobs:
                self.create_quickedit(knob_name)

            self.add_spacer()

        # Add quickedit knobs for the node saved in the node's smartEdit tab.
        knob_names = nuke_funcs.get_quickedit_knobnames(self.node)
        for knob_name in knob_names:
            self.create_quickedit(knob_name)

    def add_spacer(self):
        """Add visual spacer to separate class knobs from individual knobs."""
        line = QtWidgets.QFrame()
        line.setGeometry(QtCore.QRect(320, 150, 118, 3))
        line.setFrameShape(QtWidgets.QFrame.HLine)
        line.setFrameShadow(QtWidgets.QFrame.Sunken)

        self.view.layout_widgets.addWidget(line)

    def create_quickedit(self, knob_name):
        """Create quickedit widget by mapping the type to custom widget.

        Args:
            knob_name (str): Name of the knob to create a custom widget for.

        """
        knob_type = type(self.node.knob(knob_name))
        widget_name = knob_mapper.MAP.get(knob_type)
        factory_ = factory.KnobFactory(widget_name, self.node[knob_name])
        self.view.layout_widgets.addWidget(factory_.create())

    def create_quick_script_button(self, path):
        """Create quickscript button for the given path.

        Args:
            path (str): Absolute path of quickscript to to execute.

        """
        basename = os.path.basename(path)
        name = os.path.splitext(basename)[0]
        button = QtWidgets.QPushButton(name)
        button.clicked.connect(lambda: self.execute_quickscript(path))
        self.view.layout_widgets.addWidget(button)

    def execute_quickscript(self, path):
        """Execute the quickscript button's code.

        Args:
            path (str): Absolute path of quickscript to execute.

        """
        root, module_name = os.path.split(path)
        module_name = os.path.splitext(module_name)[0]
        module_ = imp.load_source(module_name, path)

        if not hasattr(module_, "execute"):
            message = ("The module '{}' does not contain an 'execute' "
                       "function.".format(module_name))
            dialogs.show_message_box(self.view, message)
            return

        module_.execute()

    def show_add_knob(self, selected_knob=None):
        """Show dialog for adding a new knob.

        Args:
            selected_knob (str): Name of the knob to have selected.

        """
        global _ADD_KNOB_DIALOG

        _ADD_KNOB_DIALOG = dialogs.AddKnobDialog()
        _ADD_KNOB_DIALOG.raise_()
        _ADD_KNOB_DIALOG.show()

        controller = AddKnobDialogController(_ADD_KNOB_DIALOG, self.node,
                                             selected_knob=selected_knob)

        if _ADD_KNOB_DIALOG.exec_():
            knob_name = controller.knob_name

            if knob_name in self.node_class_knobs:
                return

            knob_type = type(self.node[knob_name])
            widget_name = knob_mapper.MAP.get(knob_type)
            if not widget_name:
                message = NO_MAPPING_FOUND.format(
                    name=knob_name, type_=type(self.node.knob(knob_name)))
                dialogs.show_message_box(self.view, message)
                return

            if _ADD_KNOB_DIALOG.check_apply_to_class.isChecked():
                self.model.save_to_class_knobs(self.node.Class(), knob_name)
                self.model.reload_class_knobs()

                # Reload settings to show new global knob right away.
                self.node_class_knobs = \
                    self.model.class_knobs.get(self.node.Class()) or []

            else:
                nuke_funcs.add_quickedit_knob(self.node, controller.knob_name)

            self.populate_knobs()

    def handle_keys(self, mode):
        """Handle key pressed events for the view.

        Args:
            mode (str): Name of the mode to handle.

        """
        if mode == "launch_hotkey":
            if not self.settings["pin"]:
                self.close()

    def close(self):
        """Flush layout_widgets and close view."""
        gui_helper.flush_layout(self.view.layout_widgets)
        self.view.close()


class AddKnobDialogController():
    """Drive the AddKnob dialog."""

    def __init__(self, view, node, selected_knob):
        """Initialize the AddKnobDialogController instance.

        Args:
            view (smartEdit.mvc.dialogs.AddKnobDialog): The AddKnobDialog
                instance to drive.
            node (nuke.Node): The node to get knob names from.
            selected_knob (str): Name of the knob to have selected.

        """
        self.view = view
        self.node = node
        self.knob_name = None

        self.only_this_node = True

        self.setup_view()
        self.create_signals()

        if selected_knob:
            self.view.combo.setCurrentIndex(
                self.view.combo.findText(selected_knob))

    def setup_view(self):
        """Set up view's widgets."""
        self.view.setMinimumSize(QtCore.QSize(250, 100))
        self.view.setWindowTitle("Choose {}'s knob for quick access".format(
            self.node.name()))
        gui_helper.move_widget_under_cursor(self.view, center_window=True)
        self.view.combo.addItems(nuke_funcs.get_knobs(self.node))
        self.view.check_apply_to_class.setChecked(
            helper.load_settings()["enable_add_for_node_class"]
        )

    def create_signals(self):
        """Create signals."""
        self.view.button_add.clicked.connect(lambda: self.set())
        self.view.button_cancel.clicked.connect(lambda: self.view.reject())
        self.view.key_pressed.connect(lambda mode: self.handle_key(mode))

    def set(self):
        """Set current button."""
        self.only_this_node = self.view.check_apply_to_class.isChecked()
        self.knob_name = self.view.combo.currentText()
        self.view.accept()

    def handle_key(self, mode):
        """Handle key pressed events for the given mode.

        Args:
            mode (str): The id of the action to trigger.

        """
        if mode == "accept":
            self.set()

        elif mode == "reject":
            self.view.reject()
