"""Nuke specific functionality that requires the nuke module."""

# Import third-party modules
import nuke  # pylint: disable=import-error

# Import local modules
from smartEdit import helper
from smartEdit import knob_mapper
from smartEdit.constants import INFO_KNOB
from smartEdit.constants import NO_MAPPING_FOUND
from smartEdit.constants import REMOVE_KNOB


def get_knobs(node):
    """Return sorted list of all knob names of the given node.

    Args:
        node (nuke.Node): Node to get all knobs for.

    Returns:
        :obj:`list` of :obj:`str`: All knob names of the given node in
            alphabetical order.

    """
    ignore_knobs = [knob.lower()
                    for knob in helper.load_settings()["knobs_ignore"]]

    knobs = [name for name in node.knobs().keys()
             if not "smartEdit" in name
             and name.lower() not in ignore_knobs]

    return sorted(knobs)


def add_quickedit_knob(node, knob_name):
    """Add the given knob name as knob info to the smartEdit tab.

    Args:
        node (nuke.Node): Node to add the quickedit information to.
        knob_name (str): Name of the knob to add as quickedit.

    """
    tab = node.knob("smartEdit")
    if not tab:
        tab = nuke.Tab_Knob("smartEdit")
        node.addKnob(tab)

    knob_name_info = INFO_KNOB.format(knob_name)
    knob_remove_info = REMOVE_KNOB.format(knob_name)

    if not node.knob(knob_name_info):
        knob = nuke.String_Knob(knob_name_info, "", knob_name)
        knob.setEnabled(False)
        node.addKnob(knob)

        remove = nuke.PyScript_Knob(knob_remove_info, "remove")
        node.addKnob(remove)

        remove_command = (
            "from smartEdit import nuke_funcs; "
            "nuke_funcs.remove_quick_edit_knob(nuke.thisNode(), '{}')"
            "".format(knob_name)
        )
        remove.setCommand(remove_command)


def add_quickedit_this_knob():
    """Add quickedit knob for the currently selected knob.

    This gets run from the "Animation" menu (right click menu of a knob).

    """
    knob_name = nuke.thisKnob().name()
    node = nuke.thisNode()

    # Pre check if we have an appropriate mapping for the Nuke knob.
    if not knob_mapper.MAP.get(type(node.knob(knob_name))):
        message = NO_MAPPING_FOUND.format(name=knob_name,
                                          type_=type(node.knob(knob_name)))
        nuke.message(message)
        return

    add_quickedit_knob(node, knob_name)

    try:
        from smartEdit import smartEdit_d as smartEdit
    except ImportError:
        try:
            from smartEdit import smartEdit_w as smartEdit
        except ImportError:
            try:
                from smartEdit import smartEdit_l as smartEdit
            except ImportError:
                try:
                    from smartEdit import smartEdit_m as smartEdit
                except ImportError:
                    return

    if smartEdit._CONTROLLER:
        smartEdit._CONTROLLER.populate_knobs()


def remove_quick_edit_knob(node, knob_name):
    """Remove the quickedit knob on the node with the given name.

    Args:
        node (nuke.Node): Node to add the quickedit information to.
        knob_name (str): Name of the knob to add as quickedit.

    """
    info_knob = node.knob(INFO_KNOB.format(knob_name))
    remove_knob = node.knob(REMOVE_KNOB.format(knob_name))

    for knob in (info_knob, remove_knob):
        if knob:
            node.removeKnob(knob)

    try:
        from smartEdit import smartEdit_d as smartEdit
    except ImportError:
        try:
            from smartEdit import smartEdit_w as smartEdit
        except ImportError:
            try:
                from smartEdit import smartEdit_l as smartEdit
            except ImportError:
                try:
                    from smartEdit import smartEdit_m as smartEdit
                except ImportError:
                    return

    if smartEdit._VIEW:
        for index in reversed(range(smartEdit._VIEW.layout_widgets.count())):
            widget = smartEdit._VIEW.layout_widgets.itemAt(index).widget()
            if not hasattr(widget, "knob"):
                return
            if widget.knob.name() == knob_name:
                smartEdit._VIEW.layout_widgets.itemAt(index).widget().deleteLater()


def get_quickedit_knobnames(node):
    """Get all quickedit knob names from the given node.

    the quickedit knob names start with the constants.INFO_KNOB prefix.

    Args:
        node (nuke.Node): Name of the node to get all quickedit knobs from.

    Returns:
        :obj:`list` of :obj:`str`: All knob names that are stored as quickedit
            knobs for the given node.

    """
    prefix = INFO_KNOB.replace("{}", "")
    return sorted([name.replace(prefix, "") for name in node.knobs()
                   if name.startswith(prefix)])
