"""Abstract Knob base to be used by custom knob.

Most of the widgets that we map Nuke knobs to are derived from this base which
contains the minimal needed structure and functionality. This is an abstract
class that provides the abstract methods `create_widgets`, `create_layouts`
and `create_signals`. As these methods are highly individual to the actual
knob to map to, these methods must be implemented when deriving from this base.
In order to keep things simple, we combine view and controller here so that
we don't need to handle multiple classes, especially as these are still light
weight, quite small and don't contain too many concerns.

"""

import abc

# Import third-party modules
try:
    from PySide import QtGui as QtWidgets
except ImportError as error:
    from PySide2 import QtWidgets

# Import local modules
from smartEdit import nuke_funcs
from smartEdit.mvc import gui_helper
from smartEdit.mvc import model


class KnobBase(QtWidgets.QWidget):
    """Abstract Knob base class that provides the class skeleton.

    As the knobs are specialized to their needs and need to contain different
    widgets, the `create_widgets`, `create_layouts` and `create_signals`
    methods need to be overridden by classes that inherit from this base class.

    """

    def __init__(self, knob):
        """Initialize the Slider instance.

        Args:
            knob (nuke.Knob): The Nuke knob of a node that we want to drive.

        """
        super(KnobBase, self).__init__()

        self.knob = knob

        self.model = model.Model()

        self.advanced = self.model.load_settings()["advanced_widgets"]
        self.icons = gui_helper.load_icons()
        self.button_remove = None
        self.layout_main = QtWidgets.QHBoxLayout()
        self.setLayout(self.layout_main)

        self._build_ui()
        gui_helper.set_style_sheet(self)

    def _build_ui(self):
        """Build ui."""
        self.create_widgets()
        self.create_layouts()
        self.setup_widgets()

        self._add_remove_button()

        self.create_signals()

    def _add_remove_button(self):
        """Add remove button to UI and add signal."""
        self.button_remove = QtWidgets.QToolButton()
        gui_helper.set_icon(self.button_remove, self.icons["remove"])

        self.layout_main.addWidget(self.button_remove)

        self.button_remove.clicked.connect(self._remove)

    def _remove(self):
        """Remove the knob from the UI and from the node."""
        for index in reversed(range(self.layout_main.count())):
            try:
                self.layout_main.itemAt(index).widget().deleteLater()
            except AttributeError:
                continue

        node = self.knob.node()
        knob_name = self.knob.name()

        nuke_funcs.remove_quick_edit_knob(node, knob_name)
        model.Model().remove_class_knob(node.Class(), knob_name)


    @abc.abstractmethod
    def create_widgets(self):
        """Create widgets."""
        pass

    def setup(self):
        """Optionally set up any members."""
        pass

    def setup_widgets(self):
        """Optionally set up widgets."""
        pass

    @abc.abstractmethod
    def create_layouts(self):
        """Create layouts."""
        pass

    @abc.abstractmethod
    def create_signals(self):
        """Create signals."""
        pass
