"""Four input fields for RGBA color representation."""

# Import third-party modules
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError as error:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartEdit import helper
from smartEdit.mvc import gui_helper
from smartEdit.widgets.slider import Slider


class Color4d(Slider):
    """Four input fields for RGBA color representation."""

    # Smallest maximum value before start to generate floating sub value in
    # between for the slider.
    SMALLEST_MAX_VAL = float(
        pow(10, helper.load_settings()["slider_precision"]))

    def create_widgets(self):
        """Create widgets."""
        super(Color4d, self).create_widgets()
        
        self.label = QtWidgets.QLabel(self.knob.name())
        self.input_a = QtWidgets.QLineEdit()
        self.input_b = QtWidgets.QLineEdit()
        self.input_c = QtWidgets.QLineEdit()
        self.input_d = QtWidgets.QLineEdit()
        self.push_mode = QtWidgets.QPushButton()

        self.inputs = (
            self.input_a,
            self.input_b,
            self.input_c,
            self.input_d,
        )

        self.slider = QtWidgets.QSlider()

    def setup_widgets(self):
        """Setup widgets based on the knob's single- or multiple value mode."""
        self.slider.setOrientation(QtCore.Qt.Horizontal)
        self.push_mode.setProperty("style", "plain")

        is_single_value_mode = self.knob.singleValue()

        if is_single_value_mode:
            self.push_mode.setText("1")
            self.input_a.setProperty("style", "no_border")
            self.input_a.setText(str(self.knob.value()))
            self.slider.setMinimum(self.knob.minimum())
            self.slider.setMaximum(self.knob.maximum())
            self.slider.setRange(0, self.knob.maximum())
            self.slider.setSliderPosition(self.knob.value())
            gui_helper.set_range_style(self.slider, self.slider.value(),
                                       self.knob)

        else:
            self.push_mode.setText("4")
            self.input_a.setText(str(self.knob.value()[0]))
            self.input_a.setProperty("style", "red")

            self.input_b.setText(str(self.knob.value()[1]))
            self.input_b.setProperty("style", "green")

            self.input_c.setText(str(self.knob.value()[2]))
            self.input_c.setProperty("style", "blue")

            self.input_d.setText(str(self.knob.value()[3]))
            self.input_d.setProperty("style", "white")

        # Show inputs b, c, d; hide slider.
        for widget in (self.inputs[1:]):
            widget.setVisible(not is_single_value_mode)
        self.slider.setVisible(is_single_value_mode)
        gui_helper.set_style_sheet(self)

        if isinstance(self.knob.value(), float):
            if self.knob_maximum < self.SMALLEST_MAX_VAL:
                self.slider.setValue(self.knob.value() * self.SMALLEST_MAX_VAL)

        super(Color4d, self).setup_widgets()

    def create_layouts(self):
        """Create layouts."""
        self.layout_main.addWidget(self.label)
        self.layout_main.addWidget(self.input_a)
        self.layout_main.addWidget(self.input_b)
        self.layout_main.addWidget(self.input_c)
        self.layout_main.addWidget(self.input_d)
        self.layout_main.addWidget(self.slider)
        self.layout_main.addWidget(self.push_mode)

    def create_signals(self):
        """Create signals."""
        super(Color4d, self).create_signals()
        self.push_mode.clicked.connect(lambda: self.set_mode())
        self.input_a.returnPressed.connect(lambda: self.update())
        self.input_b.returnPressed.connect(lambda: self.update())
        self.input_c.returnPressed.connect(lambda: self.update())
        self.input_d.returnPressed.connect(lambda: self.update())

        self.slider.valueChanged.connect(lambda: self.update("slider"))

    def set_mode(self):
        """Set mode to drive 1 value or 4 separate value."""
        if self.knob.singleValue():
            value = self.knob.value()
            self.knob.setValue([value, value, value, value])
        else:
            value = self.knob.value()[0]

            self.knob.setSingleValue(True)
            self.input_a.setText(str(float(value)))

        self.setup_widgets()

    def update(self, *args):
        """Update the knob a to the current custom knob's values."""
        if self.knob.singleValue():

            if args[0] == "slider":
                if self.knob_maximum < self.SMALLEST_MAX_VAL:
                    value = self.slider.value() / self.SMALLEST_MAX_VAL
                else:
                    value = self.slider.value()
                self.input_a.setText(str(value))
            else:
                try:
                    value = float(self.input.text())
                except ValueError:
                    return
                self.slider.valueChanged.disconnect()
                self.slider.setValue(value)
                self.create_signals()

            self.knob.setValue(value)
            gui_helper.set_range_style(self.slider, value, self.knob)

        else:

            try:
                value_a = float(self.input_a.text())
                value_b = float(self.input_b.text())
                value_c = float(self.input_c.text())
                value_d = float(self.input_d.text())
            except (TypeError, ValueError):
                return

            self.knob.setValue([value_a, value_b, value_c, value_d])

            if args and args[0] == "slider":
                if self.knob_maximum < self.SMALLEST_MAX_VAL:
                    value = self.slider.value() / self.SMALLEST_MAX_VAL
                else:
                    value = self.slider.value()
                self.input.setText(str(value))
