"""Dropdown menu to choose from a defined list."""

# Import third-party modules
import nuke
try:
    from PySide import QtCore
    from PySide import QtGui
    from PySide import QtGui as QtWidgets
except ImportError as error:
    from PySide2 import QtCore
    from PySide2 import QtGui
    from PySide2 import QtWidgets

# Import local modules
from smartEdit.mvc import gui_helper
from smartEdit.widgets.base import KnobBase


class Dropdown(KnobBase):
    """Dropdown menu to choose from a defined list."""

    def create_widgets(self):
        """Create widgets."""
        self.label = QtWidgets.QLabel(self.knob.name())
        self.combo = QtWidgets.QComboBox()
        self.up = QtWidgets.QToolButton()
        self.down = QtWidgets.QToolButton()

    def setup_widgets(self):
        """Setup widgets."""
        gui_helper.set_icon(self.up, self.icons["up"])
        gui_helper.set_icon(self.down, self.icons["down"])

        self.combo.addItems(self.get_available_items())
        self.combo.setCurrentIndex(self.combo.findText(self.knob.toScript()))

    def get_available_items(self):
        """Get all available items to populate to the dropdown.

        Nuke contains several dropdown like knobs, e.g. `nuke.ChannelMask_Knob`
        and `nuke.Enumeration_Knob`. Some of these knobs contain a `values`
        method that will give us all items of the dropdown. However, not all
        nuke dropdown like knobs contain this method. In that case, we need
        to find another way to determine which elements to populate with.

        Returns:
            :obj:`list` of :obj:`string`: All items to populate the dropdown
                with being extracted either from our knob or from elsewhere.

        """
        if hasattr(self.knob, "values"):
            return self.knob.values()
        elif isinstance(self.knob, nuke.ChannelMask_Knob):
            layers = ["all"]
            layers.extend(nuke.layers())
            return layers

    def create_layouts(self):
        """Create layouts."""
        self.layout_main.addWidget(self.label)
        self.layout_main.addWidget(self.combo, 10)

        # If advanced mode show next/ previous button to cycle through the
        # list.
        if self.advanced:
            self.layout_main.addWidget(self.up, 1)
            self.layout_main.addWidget(self.down, 1)

    def create_signals(self):
        """Create signals."""
        self.down.clicked.connect(lambda: self.next_value())
        self.up.clicked.connect(lambda: self.previous_value())
        self.combo.currentIndexChanged.connect(lambda: self.update())

    def update(self):
        """Update the node's dropdown to the currently selected value."""
        self.knob.setValue(str(self.combo.currentText()))

    def next_value(self):
        """Choose the next value."""
        index = self.combo.currentIndex() + 1
        if index > self.combo.count() - 1:
            index = 0

        self.combo.setCurrentIndex(index)
        self.update()

    def previous_value(self):
        """Choose the previous value."""
        index = self.combo.currentIndex() - 1
        if index < 0:
            index = self.combo.count() - 1

        self.combo.setCurrentIndex(index)
        self.update()